package Math::3Space::Projection::Frustum;
require Math::3Space::Projection;

=pod

=encoding UTF-8

=head1 NAME

Math::3Space::Projection::Frustum - Object wrapping a 4D frustum projection, for use in OpenGL rendering

=head1 DESCRIPTION

Created by 'frustum' or 'projection' functions.

See L<Math::3Space::Projection>

=head1 AUTHOR

Michael Conrad <mike@nrdvana.net>

=head1 VERSION

version 0.008

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2024 by Michael Conrad.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut

__END__

# ABSTRACT: Object wrapping a 4D frustum projection, for use in OpenGL rendering

